/*
 * gdkglpixmap_sample.c:
 * Simple GdkGLPixmap example.
 * Save the image as out.png
 *
 * written by Roberto Huelga  <rhuelga@gmail.com>
 */

#include <stdlib.h>
#include <stdio.h>

#include <gtk/gtk.h>

#include <gtk/gtkgl.h>

#include <GL/gl.h>
#include <GL/glu.h>

unsigned int pixmap_width = 256;
unsigned int pixmap_height = 256;


void 
drawSphere()
{
	static GLfloat light_diffuse[] = {1.0, 0.0, 0.0, 1.0};
	static GLfloat light_position[] = {1.0, 1.0, 1.0, 0.0};

	glLightfv (GL_LIGHT0, GL_DIFFUSE, light_diffuse);
	glLightfv (GL_LIGHT0, GL_POSITION, light_position);
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
	glEnable (GL_DEPTH_TEST);

	glClearColor (1.0, 1.0, 1.0, 1.0);
	glClearDepth (1.0);

	glViewport (0, 0, pixmap_width, pixmap_height );

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective (40.0, 1.0, 1.0, 10.0);

	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	gluLookAt (0.0, 0.0, 3.0,
		   0.0, 0.0, 0.0,
		   0.0, 1.0, 0.0);
	glTranslatef (0.0, 0.0, -3.0);

 	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	GLUquadricObj *qobj;
	qobj = gluNewQuadric ();
	gluQuadricDrawStyle (qobj, GLU_FILL);
	gluSphere(qobj, 1.0, 20, 20);
	glEndList();

	gluDeleteQuadric( qobj );
}


int
main (int   argc,
      char *argv[])
{
	GdkGLConfig *glconfig;
	gint major, minor;

	GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *drawing_area;
	GtkWidget *button;

	/*
	 * Init GTK.
	 */

	gtk_init (&argc, &argv);

	/*
	 * Init GtkGLExt.
	 */

	gtk_gl_init (&argc, &argv);

	/*
	 * Query OpenGL extension version.
	 */

	gdk_gl_query_version (&major, &minor);
	g_print ("\nOpenGL extension version - %d.%d\n\n",
		 major, minor);

	/*
	 * Configure OpenGL-capable visual.
	 */

	/* Try double-buffered visual */
	glconfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
					      GDK_GL_MODE_DEPTH );
	if ( !glconfig) {
		g_print ("*** No appropriate OpenGL-capable visual found.\n");
		exit (1);
	}


	/*
	 * GL Pixmap
	 */
	GdkPixmap * pixmap = gdk_pixmap_new( NULL, 256, 256, 24 );
	GdkGLPixmap *gl_pixmap = gdk_pixmap_set_gl_capability( pixmap,
							       glconfig,
							       NULL );
	GdkGLDrawable * gl_drawable = gdk_pixmap_get_gl_drawable( GDK_PIXMAP( pixmap ));
	GdkGLContext * context = gdk_gl_context_new( gl_drawable,
						     NULL,
						     FALSE,
						     GDK_GL_RGBA_TYPE );

	if ( gl_drawable == NULL || context == NULL ) {
		g_printerr( "Can't create the opengl offscreen context\n");
		exit( -1 );		
	}

	if ( !gdk_gl_drawable_gl_begin( gl_drawable, context ) ) {
		g_printerr( "the offscreen opengl context don't work\n");
		exit( -1 );
	}

	drawSphere();
	
	gdk_gl_drawable_gl_end( gl_drawable );

	GdkPixbuf *pixbuf = gdk_pixbuf_get_from_drawable( NULL,
							  GDK_PIXMAP( pixmap ),
							  0,
							  0,0,0,0,
							  -1,-1);

	if ( !pixbuf ) {
		g_printerr( "Error: can't create a pixbuf from glpixmap\n" );
		exit( -1 );
	}

	GError *error = NULL;
	
	if ( !gdk_pixbuf_save( pixbuf,
			      "out.png",
			      "png",
			      &error,
			      NULL ) ){
		g_printerr( "Error: the image out.png couldn't be saved\n" );
	}
	
	
	g_object_unref( G_OBJECT( pixbuf ) );
	

	return 0;
}
